package javax.swing.plaf.nimbus;

import java.lang.ref.SoftReference;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;
import java.awt.GraphicsConfiguration;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.awt.Image;
import java.lang.ref.ReferenceQueue;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.LinkedHashMap;

class ImageCache
{
    private final LinkedHashMap<Integer, ImageCache.PixelCountSoftReference> map;
    private final int maxPixelCount;
    private final int maxSingleImagePixelSize;
    private int currentPixelCount;
    private ReadWriteLock lock;
    private ReferenceQueue<Image> referenceQueue;
    private static final ImageCache instance;
    
    static ImageCache getInstance() {
        return ImageCache.instance;
    }
    
    public ImageCache() {
        this.map = new LinkedHashMap(16, 0.75f, true);
        this.currentPixelCount = 0;
        this.lock = new ReentrantReadWriteLock();
        this.referenceQueue = new ReferenceQueue();
        this.maxPixelCount = 2097152;
        this.maxSingleImagePixelSize = 90000;
    }
    
    public ImageCache(int maxPixelCount, int maxSingleImagePixelSize) {
        this.map = new LinkedHashMap(16, 0.75f, true);
        this.currentPixelCount = 0;
        this.lock = new ReentrantReadWriteLock();
        this.referenceQueue = new ReferenceQueue();
        this.maxPixelCount = maxPixelCount;
        this.maxSingleImagePixelSize = maxSingleImagePixelSize;
    }
    
    public void flush() {
        this.lock.readLock().lock();
        try {
            this.map.clear();
        }
        finally {
            this.lock.readLock().unlock();
        }
    }
    
    public boolean isImageCachable(int n, int n2) {
        return n * n2 < this.maxSingleImagePixelSize;
    }
    
    public Image getImage(GraphicsConfiguration graphicsConfiguration, int n, int n2, Object... array) {
        this.lock.readLock().lock();
        try {
            ImageCache.PixelCountSoftReference pixelCountSoftReference = (ImageCache.PixelCountSoftReference)this.map.get(Integer.valueOf(this.hash(graphicsConfiguration, n, n2, array)));
            if (pixelCountSoftReference != null && pixelCountSoftReference.equals(graphicsConfiguration, n, n2, array)) {
                return (Image)pixelCountSoftReference.get();
            }
            return null;
        }
        finally {
            this.lock.readLock().unlock();
        }
    }
    
    public boolean setImage(Image image, GraphicsConfiguration graphicsConfiguration, int n, int n2, Object... array) {
        if (!this.isImageCachable(n, n2)) {
            return false;
        }
        int hash = this.hash(graphicsConfiguration, n, n2, array);
        this.lock.writeLock().lock();
        try {
            ImageCache.PixelCountSoftReference pixelCountSoftReference = (ImageCache.PixelCountSoftReference)this.map.get(Integer.valueOf(hash));
            if (pixelCountSoftReference != null && pixelCountSoftReference.get() == image) {
                return true;
            }
            if (pixelCountSoftReference != null) {
                this.currentPixelCount -= pixelCountSoftReference.pixelCount;
                this.map.remove(Integer.valueOf(hash));
            }
            int n3 = image.getWidth(null) * image.getHeight(null);
            this.currentPixelCount += n3;
            if (this.currentPixelCount > this.maxPixelCount) {
                ImageCache.PixelCountSoftReference pixelCountSoftReference2;
                while ((pixelCountSoftReference2 = (ImageCache.PixelCountSoftReference)this.referenceQueue.poll()) != null) {
                    this.map.remove(Integer.valueOf(pixelCountSoftReference2.hash));
                    this.currentPixelCount -= pixelCountSoftReference2.pixelCount;
                }
            }
            if (this.currentPixelCount > this.maxPixelCount) {
                Map.Entry<Object, Object> entry;
                for (Iterator iterator = this.map.entrySet().iterator(); this.currentPixelCount > this.maxPixelCount && iterator.hasNext(); this.currentPixelCount -= ((ImageCache.PixelCountSoftReference)entry.getValue()).pixelCount) {
                    entry = (Map.Entry)iterator.next();
                    iterator.remove();
                    Image image2 = (Image)((ImageCache.PixelCountSoftReference)entry.getValue()).get();
                    if (image2 != null) {
                        image2.flush();
                    }
                }
            }
            this.map.put(Integer.valueOf(hash), new ImageCache.PixelCountSoftReference(image, this.referenceQueue, n3, hash, graphicsConfiguration, n, n2, array));
            return true;
        }
        finally {
            this.lock.writeLock().unlock();
        }
    }
    
    private int hash(GraphicsConfiguration graphicsConfiguration, int n, int n2, Object... a) {
        return 31 * (31 * (31 * ((graphicsConfiguration != null) ? graphicsConfiguration.hashCode() : 0) + n) + n2) + Arrays.deepHashCode(a);
    }
    
    static {
        instance = new ImageCache();
    }
    
    private static class PixelCountSoftReference extends SoftReference<Image>
    {
        private final int pixelCount;
        private final int hash;
        private final GraphicsConfiguration config;
        private final int w;
        private final int h;
        private final Object[] args;
        
        public PixelCountSoftReference(Image referent, ReferenceQueue<? super Image> q, int pixelCount, int hash, GraphicsConfiguration config, int w, int h, Object[] args) {
            super(referent, q);
            this.pixelCount = pixelCount;
            this.hash = hash;
            this.config = config;
            this.w = w;
            this.h = h;
            this.args = args;
        }
        
        public boolean equals(GraphicsConfiguration graphicsConfiguration, int n, int n2, Object[] a) {
            return graphicsConfiguration == this.config && n == this.w && n2 == this.h && Arrays.equals(a, this.args);
        }
    }
}
